@@ -1,6 +1,4 @@
-from enum import Enum
-
-import tensorflow.keras as keras
+import warnings
 import numpy as np
 import copy
 import h5py
@@ -9,12 +7,14 @@ import json
 from pyspark.ml.param.shared import HasOutputCol, HasFeaturesCol, HasLabelCol
 from pyspark import keyword_only, RDD
 from pyspark.ml import Estimator, Model
+from pyspark.sql import DataFrame
 from pyspark.sql.types import StringType, DoubleType, StructField
 
 from tensorflow.keras.models import model_from_yaml
 from tensorflow.keras.optimizers import get as get_optimizer
 
 from .spark_model import SparkModel
+from .utils.model_utils import LossModelTypeMapper, compute_predictions
 from .utils.rdd_utils import from_vector
 from .ml.adapter import df_to_simple_rdd
 from .ml.params import *
@@ -34,6 +34,8 @@ class ElephasEstimator(Estimator, HasCategoricalLabels, HasValidationSplit, HasK
     @keyword_only
     def __init__(self, **kwargs):
         super(ElephasEstimator, self).__init__()
+        # provide default for output column, if one is not supplied using `set_` method
+        self._defaultParamMap[self.outputCol] = 'prediction'
         self.set_params(**kwargs)
 
     def get_config(self):
@@ -53,7 +55,7 @@ class ElephasEstimator(Estimator, HasCategoricalLabels, HasValidationSplit, HasK
                 'nb_classes': self.get_nb_classes(),
                 'outputCol': self.getOutputCol()}
 
-    def save(self, file_name):
+    def save(self, file_name: str):
         f = h5py.File(file_name, mode='w')
 
         f.attrs['distributed_config'] = json.dumps({
@@ -73,7 +75,7 @@ class ElephasEstimator(Estimator, HasCategoricalLabels, HasValidationSplit, HasK
     def get_model(self):
         return model_from_yaml(self.get_keras_model_config())
 
-    def _fit(self, df):
+    def _fit(self, df: DataFrame):
         """Private fit method of the Estimator, which trains the model.
         """
         simple_rdd = df_to_simple_rdd(df, categorical=self.get_categorical_labels(), nb_classes=self.get_nb_classes(),
@@ -98,11 +100,27 @@ class ElephasEstimator(Estimator, HasCategoricalLabels, HasValidationSplit, HasK
         model_weights = spark_model.master_network.get_weights()
         weights = simple_rdd.ctx.broadcast(model_weights)
         return ElephasTransformer(labelCol=self.getLabelCol(),
-                                  outputCol='prediction',
+                                  outputCol=self.getOutputCol(),
+                                  featuresCol=self.getFeaturesCol(),
                                   keras_model_config=spark_model.master_network.to_yaml(),
                                   weights=weights,
                                   loss=loss)
 
+    def setFeaturesCol(self, value):
+        warnings.warn("setFeaturesCol is deprecated in Spark 3.0.x+ - please supply featuresCol in the constructor i.e;"
+                      " ElephasEstimator(featuresCol='foo')", DeprecationWarning)
+        return self._set(featuresCol=value)
+
+    def setLabelCol(self, value):
+        warnings.warn("setLabelCol is deprecated in Spark 3.0.x+ - please supply labelCol in the constructor i.e;"
+                      " ElephasEstimator(labelCol='foo')", DeprecationWarning)
+        return self._set(labelCol=value)
+
+    def setOutputCol(self, value):
+        warnings.warn("setOutputCol is deprecated in Spark 3.0.x+ - please supply outputCol in the constructor i.e;"
+                      " ElephasEstimator(outputCol='foo')", DeprecationWarning)
+        return self._set(outputCol=value)
+
 
 def load_ml_estimator(file_name):
     f = h5py.File(file_name, mode='r')
@@ -111,7 +129,7 @@ def load_ml_estimator(file_name):
     return ElephasEstimator(**config)
 
 
-class ElephasTransformer(Model, HasKerasModelConfig, HasLabelCol, HasOutputCol):
+class ElephasTransformer(Model, HasKerasModelConfig, HasLabelCol, HasOutputCol, HasFeaturesCol):
     """SparkML Transformer implementation. Contains a trained model,
     with which new feature data can be transformed into labels.
     """
@@ -138,7 +156,7 @@ class ElephasTransformer(Model, HasKerasModelConfig, HasLabelCol, HasOutputCol):
                 'labelCol': self.getLabelCol(),
                 'outputCol': self.getOutputCol()}
 
-    def save(self, file_name):
+    def save(self, file_name: str):
         f = h5py.File(file_name, mode='w')
 
         f.attrs['distributed_config'] = json.dumps({
@@ -157,11 +175,12 @@ class ElephasTransformer(Model, HasKerasModelConfig, HasLabelCol, HasOutputCol):
         """
         output_col = self.getOutputCol()
         label_col = self.getLabelCol()
+        features_col = self.getFeaturesCol()
         new_schema = copy.deepcopy(df.schema)
         new_schema.add(StructField(output_col, StringType(), True))
 
         rdd = df.rdd.coalesce(1)
-        features = np.asarray(rdd.map(lambda x: from_vector(x.features)).collect())
+        features = np.asarray(rdd.map(lambda x: from_vector(x[features_col])).collect())
         # Note that we collect, since executing this on the rdd would require model serialization once again
         model = model_from_yaml(self.get_keras_model_config())
         model.set_weights(self.weights.value)
@@ -176,81 +195,8 @@ class ElephasTransformer(Model, HasKerasModelConfig, HasLabelCol, HasOutputCol):
         return results_df
 
 
-def load_ml_transformer(file_name):
+def load_ml_transformer(file_name: str):
     f = h5py.File(file_name, mode='r')
     elephas_conf = json.loads(f.attrs.get('distributed_config'))
     config = elephas_conf.get('config')
-    return ElephasTransformer(**config)
-
-
-class ModelType(Enum):
-    CLASSIFICATION = 1
-    REGRESSION = 2
-
-
-class _Singleton(type):
-    """ A metaclass that creates a Singleton base class when called. """
-    _instances = {}
-
-    def __call__(cls, *args):
-        if cls not in cls._instances:
-            cls._instances[cls] = super(_Singleton, cls).__call__(*args)
-        return cls._instances[cls]
-
-
-class Singleton(_Singleton('SingletonMeta', (object,), {})):
-    pass
-
-
-class LossModelTypeMapper(Singleton):
-    """
-    Mapper for losses -> model type
-    """
-    def __init__(self):
-        loss_to_model_type = {}
-        loss_to_model_type.update(
-            {'mean_squared_error': ModelType.REGRESSION,
-             'mean_absolute_error': ModelType.REGRESSION,
-             'mse': ModelType.REGRESSION,
-             'mae': ModelType.REGRESSION,
-             'cosine_proximity': ModelType.REGRESSION,
-             'mean_absolute_percentage_error': ModelType.REGRESSION,
-             'mean_squared_logarithmic_error': ModelType.REGRESSION,
-             'logcosh': ModelType.REGRESSION,
-             'binary_crossentropy': ModelType.CLASSIFICATION,
-             'categorical_crossentropy': ModelType.CLASSIFICATION,
-             'sparse_categorical_crossentropy': ModelType.CLASSIFICATION})
-        self.__mapping = loss_to_model_type
-
-    def get_model_type(self, loss):
-        return self.__mapping.get(loss)
-
-    def register_loss(self, loss, model_type):
-        if callable(loss):
-            loss = loss.__name__
-        self.__mapping.update({loss: model_type})
-
-
-def compute_predictions(model, model_type, rdd, features):
-    predict_function = determine_predict_function(model, model_type)
-    predictions = rdd.ctx.parallelize(predict_function(features)).coalesce(1)
-    if model_type == ModelType.CLASSIFICATION:
-        predictions = predictions.map(lambda x: tuple(str(x)))
-    else:
-        predictions = predictions.map(lambda x: tuple([float(x)]))
-    results_rdd = rdd.zip(predictions).map(lambda x: x[0] + x[1])
-    return results_rdd
-
-
-def determine_predict_function(model,
-                               model_type):
-    if model_type == ModelType.CLASSIFICATION:
-        if isinstance(model, keras.models.Sequential):
-            predict_function = model.predict_classes
-        else:
-            # support for functional API
-            predict_function = lambda x: model.predict(x).argmax(axis=-1)
-    else:
-        predict_function = model.predict
-
-    return predict_function
+    return ElephasTransformer(**config)
\ No newline at end of file
