@@ -1,13 +1,21 @@
+from functools import partial
+from itertools import tee
+from typing import Union
+
 import pyspark
 import h5py
 import json
+
+import numpy as np
+from tensorflow.keras.models import model_from_yaml
+from pyspark import RDD
 from tensorflow.keras.optimizers import serialize as serialize_optimizer
 from tensorflow.keras.optimizers import get as get_optimizer
 from tensorflow.keras.models import load_model
 
 from .parameter.factory import ClientServerFactory
 from .utils import subtract_params
-from .utils import lp_to_simple_rdd
+from .utils import lp_to_simple_rdd, to_simple_rdd
 from .utils import model_to_dict
 from .mllib import to_matrix, from_matrix, to_vector, from_vector
 from .worker import AsynchronousSparkWorker, SparkWorker
@@ -15,8 +23,8 @@ from .worker import AsynchronousSparkWorker, SparkWorker
 
 class SparkModel(object):
 
-    def __init__(self, model, mode='asynchronous', frequency='epoch',  parameter_server_mode='http', num_workers=None,
-                 custom_objects=None, batch_size=32,  port=4000, *args, **kwargs):
+    def __init__(self, model, mode='asynchronous', frequency='epoch', parameter_server_mode='http', num_workers=None,
+                 custom_objects=None, batch_size=32, port=4000, *args, **kwargs):
         """SparkModel
 
         Base class for distributed training on RDDs. Spark model takes a Keras
@@ -62,7 +70,7 @@ class SparkModel(object):
         self.serialized_model = model_to_dict(model)
         if self.mode is not 'synchronous':
             factory = ClientServerFactory.get_factory(self.parameter_server_mode)
-            self.parameter_server = factory.create_server(self.serialized_model, self.port, mode=self.mode)
+            self.parameter_server = factory.create_server(self.serialized_model, self.port, self.mode)
             self.client = factory.create_client(self.port)
 
     def get_config(self):
@@ -103,17 +111,22 @@ class SparkModel(object):
     def stop_server(self):
         self.parameter_server.stop()
 
-    def predict(self, data):
+    def predict(self, data: Union[RDD, np.array]):
         """Get prediction probabilities for a numpy array of features
         """
-        return self._master_network.predict(data)
+        if isinstance(data, (np.ndarray,)):
+            from pyspark.sql import SparkSession
+            sc = SparkSession.builder.getOrCreate().sparkContext
+            data = sc.parallelize(data)
+        return self._predict(data)
 
-    def predict_classes(self, data):
-        """ Predict classes for a numpy array of features
-        """
-        return self._master_network.predict_classes(data)
+    def evaluate(self, x_test, y_test):
+        from pyspark.sql import SparkSession
+        sc = SparkSession.builder.getOrCreate().sparkContext
+        test_rdd = to_simple_rdd(sc, x_test, y_test)
+        return self._evaluate(test_rdd)
 
-    def fit(self, rdd, **kwargs):
+    def fit(self, rdd: RDD, **kwargs):
         """
         Train an elephas model on an RDD. The Keras model configuration as specified
         in the elephas model is sent to Spark workers, abd each worker will be trained
@@ -135,7 +148,7 @@ class SparkModel(object):
             raise ValueError(
                 "Choose from one of the modes: asynchronous, synchronous or hogwild")
 
-    def _fit(self, rdd, **kwargs):
+    def _fit(self, rdd: RDD, **kwargs):
         """Protected train method to make wrapping of modes easier
         """
         self._master_network.compile(optimizer=get_optimizer(self.master_optimizer),
@@ -176,6 +189,41 @@ class SparkModel(object):
         if self.mode in ['asynchronous', 'hogwild']:
             self.stop_server()
 
+    def _predict(self, rdd: RDD):
+        if self.num_workers:
+            rdd = rdd.repartition(self.num_workers)
+        yaml_model = self.master_network.to_yaml()
+        weights = self.master_network.get_weights()
+        weights = rdd.context.broadcast(weights)
+
+        def _predict(model, data):
+            model = model_from_yaml(model)
+            model.set_weights(weights.value)
+            data = np.array([x for x in data])
+            return model.predict(data)
+        predictions = rdd.mapPartitions(partial(_predict, yaml_model)).collect()
+        return predictions
+
+    def _evaluate(self, rdd: RDD):
+        yaml_model = self.master_network.to_yaml()
+        optimizer = self.master_optimizer
+        loss = self.master_loss
+        weights = self.master_network.get_weights()
+        weights = rdd.context.broadcast(weights)
+
+        def _evaluate(model, optimizer, loss, data_iterator):
+            model = model_from_yaml(model)
+            model.set_weights(weights.value)
+            model.compile(optimizer, loss)
+            feature_iterator, label_iterator = tee(data_iterator, 2)
+            x_test = np.asarray([x for x, y in feature_iterator])
+            y_test = np.asarray([y for x, y in label_iterator])
+            return [model.evaluate(x_test, y_test)]
+        if self.num_workers:
+            rdd = rdd.repartition(self.num_workers)
+        results = rdd.mapPartitions(partial(_evaluate, yaml_model, optimizer, loss)).mean()
+        return results
+
 
 def load_spark_model(file_name):
     model = load_model(file_name)
