@@ -1,7 +1,7 @@
-from tensorflow.keras.models import model_from_json
+from tensorflow.keras.models import model_from_json, Model
 
 
-def model_to_dict(model):
+def model_to_dict(model: Model):
     """Turns a Keras model into a Python dictionary
 
     :param model: Keras model instance
@@ -10,13 +10,13 @@ def model_to_dict(model):
     return dict(model=model.to_json(), weights=model.get_weights())
 
 
-def dict_to_model(dict):
+def dict_to_model(_dict: dict):
     """Turns a Python dictionary with model architecture and weights
     back into a Keras model
 
-    :param dict: dictionary with `model` and `weights` keys.
+    :param _dict: dictionary with `model` and `weights` keys.
     :return: Keras model instantiated from dictionary
     """
-    model = model_from_json(dict['model'])
-    model.set_weights(dict['weights'])
+    model = model_from_json(_dict['model'])
+    model.set_weights(_dict['weights'])
     return model
